-------------------------------------------------------------------------------
--                G  O  B  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

local addonName, L = ...;

-------------------------------------------------------------------------------
--                L  O  C  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

-- Mapping of spell ID to trade skill ID
local gatherSpellSkill =
{
  -- Archaeology
  [ 73979] = 794,
  -- Engineering
  [ 49383] = 202,
  -- Fishing
  [  7620] = 356,
  [  7731] = 356,
  [  7732] = 356,
  [ 18248] = 356,
  [ 33095] = 356,
  [ 51294] = 356,
  [ 88868] = 356,
  [110410] = 356,
  [131474] = 356,
  [131476] = 356,
  [131490] = 356,
  -- Herbalism
  [  2366] = 182,
  [  2368] = 182,
  [  3570] = 182,
  [ 11993] = 182,
  [ 28695] = 182,
  [ 50300] = 182,
  [ 74519] = 182,
  [110413] = 182,
  -- Mining
  [  2575] = 186,
  [  2576] = 186,
  [  3564] = 186,
  [ 10248] = 186,
  [ 29354] = 186,
  [ 50310] = 186,
  [ 74517] = 186,
  [102161] = 186,
  -- Skinning
  [  8613] = 393,
  [  8617] = 393,
  [  8618] = 393,
  [ 10768] = 393,
  [ 32678] = 393,
  [ 50305] = 393,
  [ 74522] = 393,
  [102216] = 393,
};

-- Info about the gathering skills
local gatherSkillInfo =
{
  [794] = -- Archaeology
  {
    index   = 0,
    tooltip = L["Requires Archaeology"]
  },
  [202] = -- Engineering
  {
    index   = 0,
    tooltip = UNIT_SKINNABLE_BOLTS
  },
  [356] = -- Fishing
  {
    index   = 0,
    tooltip = ""
  },
  [182] = -- Herbalism
  {
    index   = 0,
    tooltip = UNIT_SKINNABLE_HERB
  },
  [186] = -- Mining
  {
    index   = 0,
    tooltip = UNIT_SKINNABLE_ROCK
  },
  [393] = -- Skinning
  {
    index   = 0,
    tooltip = UNIT_SKINNABLE_LEATHER
  },
};

--
-- These value have been copied from 'Blizzard_TradeSkillUI.lua' because the
-- tradeskill panel is implemented as an on-demand addon.
--
local tradeSkillTypeInfo =
{
  ["trivial"] = {r = 0.50, g = 0.50, b = 0.50},
  ["easy"]    = {r = 0.25, g = 0.75, b = 0.25},
  ["medium"]  = {r = 1.00, g = 1.00, b = 0.00},
  ["optimal"] = {r = 1.00, g = 0.50, b = 0.25},
};

local tradeSkillInfo =
{
  frame          = nil,
  isRegistered   = false,
  maxRank        = 0,
  numProfessions = 0,
  numMaxed       = 0,
};

local recipeInfo =
{
  index     = 0,
  spellId   = 0,
  skillType = "",
  rank      = 0,
  maxRank   = 0,
  skillName = "",
};

-------------------------------------------------------------------------------
--                L  O  C  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function hooked to 'DoTradeSkill' to capture the spell ID of the item being
-- created so that its skill type can be obtained for each iteration
--
local function tradeSkillDoRecipe(idx, count)
  -- Local Variables
  local spellId;

  -- Extract the spell ID from the recipe's link
  spellId = string.gsub(GetTradeSkillRecipeLink(idx),
    "^[^:]*:([^|]*)\|h.*$", "%1");
  if (spellId ~= nil) then
    -- Save the spell ID
    recipeInfo.index     = tonumber(idx);
    recipeInfo.spellId   = tonumber(spellId);
    recipeInfo.skillName = GetTradeSkillLine();
  else
    recipeInfo.index     = 0;
    recipeInfo.spellId   = 0;
    recipeInfo.skillName = "";
  end

  return;
end

--
-- Function hooked to 'SolveArtifact' to capture the spell ID of the artifact
-- being solved.
--
local function tradeSkillSolveArtifact(...)
  -- Local Variables
  local artifactName = GetSelectedArtifactInfo();

  -- Check the artifact's name could be obtained
  if (artifactName ~= nil) then
    -- Get the spell ID for the artifact
    recipeInfo.index     = 0;
    recipeInfo.spellId   = NigglesArchaeologyGetSpellId(artifactName);
    recipeInfo.skillType = "optimal";
    recipeInfo.skillName, _, recipeInfo.rank, recipeInfo.maxRank =
      GetProfessionInfo(gatherSkillInfo[794].index);
  end

  return;
end

--
-- Function to get the skill type from the GameTooltip when a world object is
-- gathered.  --
local function tradeSkillTypeFromTooltip(skillId)
  -- Local Variables
  local colourR;
  local colourG;
  local colourB;
  local skillType = "optimal";

  -- Process each line of the Game Tooltip
  for idx = GameTooltip:NumLines(), 1, -1 do
    -- Check if the tooltip line matches the gather skill's tooltip
    if (_G["GameTooltipTextLeft"..idx]:GetText() ==
         gatherSkillInfo[skillId].tooltip) then
      -- Get the colour of the tooltip line
      colourR, colourG, colourB = _G["GameTooltipTextLeft"..idx]:GetTextColor();

      -- Find the skill type, based on the colour
      for currentType, currentInfo in pairs(tradeSkillTypeInfo) do
        if ((math.abs(colourR-currentInfo.r) < 0.01) and
            (math.abs(colourG-currentInfo.g) < 0.01) and
            (math.abs(colourB-currentInfo.b) < 0.01)) then
          skillType = currentType;
        end
      end
      break;
    end
  end

  return skillType;
end

--
-- Function to process events
--
local function tradeSkillOnEvent(self, event, ...)
  -- Local Variables
  local skillId;

  -- Process the event
  if ((event == "UNIT_SPELLCAST_START") or
      (event == "UNIT_SPELLCAST_CHANNEL_START")) then
    -- Local Variables
    local skillId;

    -- Get the spell ID
    local _, _, _, _, spellId = ...;
    skillId = gatherSpellSkill[spellId];

    -- Check if a gathering skill is being cast
    if ((skillId ~= nil) and (gatherSkillInfo[skillId] ~= nil)) then
      -- Get the skill type from the tooltip
      recipeInfo.skillType = tradeSkillTypeFromTooltip(skillId);

      -- Get the profession's current and max ranks
      recipeInfo.skillName, _, recipeInfo.rank, recipeInfo.maxRank =
        GetProfessionInfo(gatherSkillInfo[skillId].index);
    elseif (spellId == recipeInfo.spellId) then
      -- Get the skill type
      if (recipeInfo.index > 0) then
        -- Get the skill type
        _, recipeInfo.skillType = GetTradeSkillInfo(recipeInfo.index);
        _, recipeInfo.rank, recipeInfo.maxRank = GetTradeSkillLine();
      else
        recipeInfo.skillType = "optimal";
      end
    else
      recipeInfo.skillType = "";
    end

    -- Check if a warning should be displayed
    if ((recipeInfo.skillType ~= "") and
        (recipeInfo.skillType ~= "trivial") and
        (recipeInfo.maxRank < tradeSkillInfo.maxRank) and
        (recipeInfo.rank >= recipeInfo.maxRank-5)) then
      if (recipeInfo.rank >= recipeInfo.maxRank) then
        Niggles:AddWarning(L["Trade Skill Capped"], recipeInfo.skillName);
      else
        Niggles:AddWarning(L["Trade Skill Near Cap"], recipeInfo.skillName);
      end
      PlaySound("RaidWarning");
    end
  elseif (event == "SKILL_LINES_CHANGED") then
    -- Initialise some variables
    local skillRank;
    local professions = {};
    tradeSkillInfo.numProfessions = 0;
    tradeSkillInfo.numMaxed = 0;

    -- Get the player's professions
    professions[1], professions[2], professions[3],
    professions[4], professions[5], professions[6] = GetProfessions();

    -- Get the index of each gather professions the player knows
    for _, professionIdx in pairs(professions) do
      -- Get info for the profession
      _, _, skillRank, _, _, _, skillId = GetProfessionInfo(professionIdx);

      -- Check if the skill is a gather skill
      if (gatherSkillInfo[skillId] ~= nil) then
        -- Save the profession's skill index
        gatherSkillInfo[skillId].index = professionIdx;
      end

      -- Check if the skill is at the maximum rank possible
      tradeSkillInfo.numProfessions = tradeSkillInfo.numProfessions+1;
      if (skillRank == tradeSkillInfo.maxRank) then
        tradeSkillInfo.numMaxed = tradeSkillInfo.numMaxed+1;
      end
    end

    -- Check if there are any professions that aren't maxed
    if (tradeSkillInfo.numMaxed < tradeSkillInfo.numProfessions) then
      if (not tradeSkillInfo.isRegistered) then
        tradeSkillInfo.frame:RegisterUnitEvent("UNIT_SPELLCAST_START",
          "player");
        tradeSkillInfo.frame:RegisterUnitEvent("UNIT_SPELLCAST_CHANNEL_START",
          "player");
        tradeSkillInfo.isRegistered = true;
      end
    else
      if (tradeSkillInfo.isRegistered) then
        tradeSkillInfo.frame:UnregisterEvent("UNIT_SPELLCAST_START");
        tradeSkillInfo.frame:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_START");
        tradeSkillInfo.isRegistered = false;
      end
    end
  end

  return;
end

-------------------------------------------------------------------------------
--               G  L  O  B  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to enable/disable the trade skill features
--
function NigglesTradeSkillEnable()
  -- Check if the frame used to monitor events has already be created
  if ((tradeSkillInfo.frame == nil) and
      (NigglesDBC.settings.WarningTradeSkillCap)) then
    -- Initialise some info
    tradeSkillInfo.maxRank = PROFESSION_RANKS[#PROFESSION_RANKS][1];

    -- Create the frame used to monitor events
    tradeSkillInfo.frame = CreateFrame("Frame", nil, UIParent);
    tradeSkillInfo.frame:SetScript("OnEvent", tradeSkillOnEvent);

    -- Hook the function that does trade skill recipes
    hooksecurefunc("DoTradeSkill", tradeSkillDoRecipe);
    hooksecurefunc("SolveArtifact", tradeSkillSolveArtifact);
  end

  -- (Un)register for events
  if (tradeSkillInfo.frame ~= nil) then
    if (NigglesDBC.settings.WarningTradeSkillCap) then
      tradeSkillInfo.frame:RegisterEvent("SKILL_LINES_CHANGED");
      tradeSkillOnEvent(tradeSkillInfo.frame, "SKILL_LINES_CHANGED");
    else
      tradeSkillInfo.frame:UnregisterEvent("UNIT_SPELLCAST_START");
      tradeSkillInfo.frame:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_START");
      tradeSkillInfo.frame:UnregisterEvent("SKILL_LINES_CHANGED");
      tradeSkillInfo.isRegistered = false;
    end
  end

  return;
end
